// Copyright 1997-1999 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

#import "ONPortAddress.h"

#import <Foundation/Foundation.h>
#import <OmniBase/OmniBase.h>
#import <OmniBase/system.h>

// For the OFEncode and OFDecode macros -- we don't need to link OmniFoundation though.
#import <OmniFoundation/OFUtilities.h>

#import "ONHost.h"
#import "ONHostAddress.h"

RCS_ID("$Header: /Network/Developer/Source/CVS/OmniGroup/OmniNetworking/ONPortAddress.m,v 1.8 1999/03/30 21:25:20 kc Exp $")

@implementation ONPortAddress

- initWithHost:(ONHost *)aHost portNumber:(unsigned short int)port;
{
    NSArray *hostAddresses;

    hostAddresses = [aHost addresses];
    if ([hostAddresses count] == 0)
        [NSException raise:ONInternetSocketConnectFailedExceptionName
                    format:@"Unable to create port address: no IP address for host '%@'", [aHost hostname]];
    return [self initWithHostAddress:[hostAddresses objectAtIndex:0]
                          portNumber:port];
}

- initWithHostAddress:(ONHostAddress *)hostAddress portNumber:(unsigned short int)port;
{
    if (![super init])
        return nil;

    portAddress = (struct sockaddr_in *)NSZoneMalloc(NULL, sizeof(struct sockaddr_in));
    portAddress->sin_family = AF_INET;
    if (hostAddress)
	portAddress->sin_addr = *[hostAddress internetAddress];
    else
	portAddress->sin_addr.s_addr = htonl(INADDR_ANY);
    portAddress->sin_port = htons(port);

    return self;
}

- (void)dealloc;
{
    NSZoneFree(NULL, portAddress);
    [super dealloc];
}

- (const struct sockaddr_in *)portAddress;
{
    return portAddress;
}

- (const struct in_addr *)hostAddress;
{
    return &(portAddress->sin_addr);
}

- (unsigned long int)hostNumber;
{
    return ntohl(portAddress->sin_addr.s_addr);
}

- (unsigned short int)portNumber;
{
    return ntohs(portAddress->sin_port);
}

- (BOOL) isMulticastAddress;
{
    // Either inet_addr returns addresses in host byte order
    // or this macro expects them in host byte order.  It is
    // not clear from the documentation (such as it is).
    return IN_MULTICAST(ntohl(portAddress->sin_addr.s_addr));
}

//
// NSCoding protocol
//

- (id) initWithCoder: (NSCoder *) coder;
{
    [super init];

    portAddress = (struct sockaddr_in *)NSZoneMalloc(NULL, sizeof(struct sockaddr_in));
    portAddress->sin_family = AF_INET;

    // These two fields are always stored in network byte order
    // internally, so we don't need to swap them.
    OFDecode(coder, portAddress->sin_addr.s_addr);
    OFDecode(coder, portAddress->sin_port);

    return self;
}

- (void) encodeWithCoder: (NSCoder *) coder;
{
    // These two fields are always stored in network byte order
    // internally, so we don't need to swap them.
    OFEncode(coder, portAddress->sin_addr.s_addr);
    OFEncode(coder, portAddress->sin_port);
}

@end
